
import java.io.IOException;
import java.io.PrintWriter;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Die Klasse SAXSchreiber erweitert den
 * SAX DefaultHandler zur Ausgabe der XML Dateien
 * auf einen PrintWriter.
 * JDK &gt;= 1.4
 */

public class SAXSchreiber extends DefaultHandler {

    PrintWriter pw = null;

    public SAXSchreiber() {
	pw = new Screen();
    }

    public SAXSchreiber(PrintWriter p) {
	pw = p;
    }


    public void startDocument() {
        pw.println("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
        pw.flush();
    }

    public void endDocument() {
        pw.println();
        pw.flush();
    }


    public void startElement(String nameuri, String local, String quali, 
                             Attributes attrs) {
        pw.print('<');
        pw.print(local);
        if (attrs != null) {
            int len = attrs.getLength();
            for (int i = 0; i < len; i++) {
                pw.print(' ');
                pw.print(attrs.getQName(i));
                pw.print("=\"");
                pw.print(attrs.getValue(i));
                pw.print('"');
            }
        }
        pw.print('>');
        pw.flush();
    } 

    public void endElement(String nameuri, String local, String quali) {
        pw.print("</");
        pw.print(local);
        pw.print('>');
        pw.flush();
    } 

    public void characters(char ch[], int start, int length) {
        pw.print( new String(ch, start, length) );
        pw.flush();
    } 

    public void ignorableWhitespace(char ch[], int start, int length) {
        characters(ch, start, length);
    } 


    public void processingInstruction(String target, String data) {
        pw.print("<?");
        pw.print(target);
        if (data != null && data.length() > 0) {
            pw.print(' ');
            pw.print(data);
        }
        pw.print("?>");
        pw.flush();
    }

    public void warning(SAXParseException ex) {
        pw.println("\n<!-- [Warning] "+
                   ex.getSystemId()+": line "+
                   ex.getLineNumber()+", column "+
                   ex.getColumnNumber()+":\n"+
                   ex.getMessage() + " -->");
    }

    public void error(SAXParseException ex) {
        pw.println("\n<!-- [Error] "+
                   ex.getSystemId()+": line "+
                   ex.getLineNumber()+", column "+
                   ex.getColumnNumber()+":\n"+
                   ex.getMessage() + " -->");
    }

    public void fatalError(SAXParseException ex) {
        pw.println("\n<!-- [Fatal Error] "+
                   ex.getSystemId()+": line "+
                   ex.getLineNumber()+", column "+
                   ex.getColumnNumber()+":\n"+
                   ex.getMessage() + " -->");
    }


    public InputSource resolveEntity(String publicId,
                                     String systemId) {
        pw.println("\n<!-- [resolveEntity] publicId: "+
                   publicId + ", systemId: " +
                   systemId + " -->" );
        boolean dtd = false;
        boolean pid = false;
        boolean sid = false;
        if ( ( publicId != null ) || ( systemId != null ) ) {
	   pid = pid || ( publicId.indexOf("DTD") >= 0 );
	   pid = pid || ( publicId.indexOf("dtd") >= 0 );
	   sid = sid || ( systemId.indexOf("DTD") >= 0 );
	   sid = sid || ( systemId.indexOf("dtd") >= 0 );
	}
        if (pid) {
           pw.print("<!DOCTYPE ???? " );
           pw.print("PUBLIC \"" + publicId + "\"");
           if (sid) {
              pw.print(" \"" + systemId + "\"");
	      pw.println(" >");
	   }
	} else if (sid) {
           pw.print("<!DOCTYPE ???? " );
           pw.print("SYSTEM \"" + systemId + "\"");
           pw.println(" >");
	}
	return null;
    }

    public void notationDecl(String name, String publicId,
                                     String systemId) {
        pw.println("\n[notationDeclaration] name: " + name + " publicId: "+
                   publicId + ", systemId: " +
                   systemId );
    }

}
